/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <stdio.h>

#include "netutil/NetworkUtilityFunctions.h"
#include "support/PortableMacros.h"
#include "support/PortableStatusConstants.h"

#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>

int Connect(const char * hostName, int port, const char * debugTitle, bool errorsOnly)
{
   if ((debugTitle)&&(errorsOnly == false)) 
   {
      LogTime(MUSCLE_LOG_INFO, "%s: Connecting to [%s:%i]: ", debugTitle, hostName, port);
      fflush(stdout);
   }
   struct hostent * he = gethostbyname(hostName);
   if (he)
   {
      int s = socket(AF_INET, SOCK_STREAM, 0);
      if (s >= 0)
      {
         struct sockaddr_in saAddr;
         memset(&saAddr, 0, sizeof(saAddr));
         saAddr.sin_family = AF_INET;
         saAddr.sin_port   = htons(port);
         memcpy(&saAddr.sin_addr.s_addr, he->h_addr, he->h_length);

         if (connect(s, (struct sockaddr *) &saAddr, sizeof(saAddr)) >= 0)
         {
            if ((debugTitle)&&(errorsOnly == false)) Log(MUSCLE_LOG_INFO, "Connected!\n");
            return s;
         }
         else if (debugTitle) 
         {
            if (errorsOnly) Log(MUSCLE_LOG_INFO, "%s: connect() to [%s:%i] failed!\n", debugTitle, hostName, port);
                       else Log(MUSCLE_LOG_INFO, "Connection failed!\n");
         }
         closesocket(s);
      }
      else if (debugTitle) 
      {
         if (errorsOnly) Log(MUSCLE_LOG_INFO, "%s: socket() failed!\n", debugTitle);
                    else Log(MUSCLE_LOG_INFO, "socket() failed!\n");
      }
   }
   else if (debugTitle) 
   {
      if (errorsOnly) Log(MUSCLE_LOG_INFO,"%s: Couldn't lookup hostname [%s]\n", debugTitle, hostName);
                 else Log(MUSCLE_LOG_INFO,"hostname lookup failed!\n");
   }
   return -1;
}

int CreateAcceptSocket(int /*listenPort*/)
{
   return -1;  // TODO: implement this
}

void Inet_NtoA(uint32 addr, char * buf)
{
   sprintf(buf, "%li.%li.%li.%li", (addr>>24)&0xFF, (addr>>16)&0xFF, (addr>>8)&0xFF, (addr>>0)&0xFF);
}

status_t GetPeerInfo(int socket, uint32 & setIPAddress, uint16 & setPort)
{
   struct sockaddr_in saTempAdd;
#ifdef __BEOS__
   int length = sizeof(sockaddr_in);
#else
   size_t length = sizeof(sockaddr_in);
#endif
   if (getpeername(socket, (struct sockaddr *)&saTempAdd, &length) == 0)
   {
      setIPAddress = ntohl(saTempAdd.sin_addr.s_addr);
      setPort = ntohs(saTempAdd.sin_port);
      return B_NO_ERROR;
   }
   return B_ERROR;
}
