/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */

//! author="Jeremy Friesner"

#ifndef NETWORK_UTILITY_FUNCTIONS_H
#define NETWORK_UTILITY_FUNCTIONS_H

#include "netutil/TimeUtilityFunctions.h"

//:Convenience method for connecting with TCP to a given hostName/port.
// Returns the sockfd if the connection is successful, -1 otherwise.
// If (debugTitle) is set, debug output will be enabled and debug output
// will be prefaced by this string.
// If (debugOutputOnErrorsOnly) is true, debug output will be printed
// only if an error condition occurs.
int Connect(const char * hostName, int port, const char * debugTitle = NULL, bool debugOutputOnErrorsOnly = true);

//:Returns a socket fd that is ready to accept TCP connections on the given port, or -1 on failure.
// (you may pass this socket to select() or accept()...).  If (listenPort) is zero, the system will
// choose a port to listen on.
// Note:  NOT IMPLEMENTED YET!
int CreateAcceptSocket(int listenPort);

// Returns a string representation of the IP address (address).  Useful on OS's where inet_ntoa
// isn't available.  (outBuf) should point to a buffer at least 13 bytes long.
void Inet_NtoA(uint32 address, char * outBuf);

// fills out (retHostIP) and (retPort) with information about the
// remote peer on the other end of (socket).  Returns B_NO_ERROR
// on success, B_ERROR on failure (such as if the socket isn't connected).
status_t GetPeerInfo(int socket, uint32 & retHostIP, uint16 & retPort);

#endif
